<?php
use PHPUnit\Framework\TestCase;

abstract class ClientTestBase extends TestCase {
	/** @var  \DEPOClient\ClientInterface */
	protected $client;

	public function testGetPlaces() {
		$places = $this->client->getPlaces();
		$this->assertInternalType('array', $places);
		foreach($places as $place) {
			$this->assertInstanceOf(\DEPOClient\Entities\Place::class, $place);
		}
	}

	public function testGetPackages() {
		$packages = $this->client->getPackages();
		$this->assertInternalType('array', $packages);
		foreach($packages as $package) {
			$this->assertInstanceOf(\DEPOClient\Entities\Package::class, $package);
		}
	}
	
	public function testGetNotExistingPackage() {
		$this->expectException(\DEPOClient\Exceptions\DEPOException::class);
		$number = 'obviously-not-existing-id';
		$package = $this->client->getPackage($number);
		$this->assertInstanceOf(\DEPOClient\Entities\Package::class, $package);
		$this->assertEquals($number, $package->number);
	}

	public function testGenerateLabelForExistingPackage() {
		$label = new \DEPOClient\Entities\Label();
		$label->number = '2000000047294';
		$label->size = '2x2';
		$this->client->generateLabel($label);
		$this->assertFileExists($label->pdf_filename);
	}

	public function testGenerateLabelForNotExistingPackage() {
		$this->expectException(\DEPOClient\Exceptions\DEPOException::class);
		$label = new \DEPOClient\Entities\Label();
		$label->number = 'obviously-not-existing-id';
		$label->size = '3x5';
		$this->client->generateLabel($label);
	}

	public function testGenerateLabelWrongSize() {
		$this->expectException(\DEPOClient\Exceptions\DEPOException::class);
		$label = new \DEPOClient\Entities\Label();
		$label->number = '2000000047294';
		$label->size = '100x100';
		$this->client->generateLabel($label);
	}

	public function testSendPackage() {
		$package = new \DEPOClient\Entities\Package();

		// fill required fields
		$package->target = '31703542';
		$package->insurance = 100;
		$package->recipient_name = 'Josef Válek';
		$package->recipient_phone = '00420774874302';
		$package->recipient_email = 'valek@1webit.cz';

		$this->client->sendPackage($package);

		$this->assertNotNull($package->getId());

		return $package->getId();
	}

	/**
	 * @depends testSendPackage
	 */
	public function testGetExistingPackage($number) {
		$package = $this->client->getPackage($number);
		$this->assertInstanceOf(\DEPOClient\Entities\Package::class, $package);
		$this->assertEquals($number, $package->getId());
		return $package->getId();
	}

	/**
	 * @depends testGetExistingPackage
	 */
	public function testCancelPackage($number) {
		$this->client->cancelPackage($number);
	}

	public function testSendInvalidPackage() {
		$this->expectException(\DEPOClient\Exceptions\InvalidEntityException::class);
		$package = new \DEPOClient\Entities\Package();

		// fill required fields
		$package->target = '31703542';
		$package->insurance = 100;
		$package->recipient_name = 'Josef Válek';
		$package->recipient_phone = '00420774874302abc';
		$package->recipient_email = 'valek@1webit.cz';

		$this->client->sendPackage($package);

	}


}
