<?php

/**
 * Class AdminDepoCarrierController
 * 
 * Required parameters:
 *  - id_order
 *
 * All actions (except PDF are supposed to be ajax)
 * use &ajax&action=method_name to call ajaxProcessMethodName()
 */
class AdminDepoCarrierController extends ModuleAdminController {
	/** @var  DEPOCarrier */
	public $module;
	
	/** @var  int Current order id, set in postProcess*/
	protected $id_order;

	/**
	 * Assigns id_order and calls parent.
	 */
	public function postProcess() {
		$this->id_order = Tools::getValue('id_order', false);
		if ($this->id_order === false) {
			die('Missing id_order');
		}
		parent::postProcess();
	}

	/**
	 * Attempts to send a package.
	 * @throws PrestaShopException
	 */
	public function ajaxProcessSendPackage() {
		if ($this->checkToken()) {
			/** @var OrderCore $order */
			$order = new Order($this->id_order);
			$order_depo = new OrderDepoExt($this->id_order);

			if (!Validate::isLoadedObject($order) || !Validate::isLoadedObject($order_depo)) {
				die('Invalid order or orderDepo');
			}

			$result = $this->module->sendPackage($order, $order_depo);
			$flash = $result ?
				['type' => 'success', 'message' => 'DEPO_SENT'] :
				['type' => 'warning', 'message' => 'DEPO_NOT_SENT'];
			
			die(json_encode([
				'result' => $result ? 'success' : 'fail',
				'snippets' => self::renderSnippets($order_depo, [$flash])
			]));
		}
	}
	
	public function ajaxProcessDepoDetail() {
		$order_depo = new OrderDepoExt($this->id_order);

		try {
			$client = $this->module->getClient();
			$package = $client->getPackage(Tools::getValue('package_number', $order_depo->number));
			$place = $client->getPlace($package->target);
			$this->context->smarty->assign([
				'depo_detail' => [
					'package' => $package,
					'place' => $place
				]
			]);
			$result = TRUE;
		} catch(\DEPOClient\Exceptions\DEPOException $e) {
			$result = FALSE;
		}

		$flashes = [];

		die(json_encode([
			'result' => $result ? 'success' : 'fail',
			'snippets' => self::renderSnippets($order_depo, $flashes)
		]));
	}

	/**
	 * Renders tab nav and tab content for admin order view.
	 * 
	 * @param OrderDepoExt $order_depo
	 * @param array $flashes
	 * @return array
	 */
	public static function renderSnippets(OrderDepoExt $order_depo, $flashes = []) {
		// get smarty
		$smarty = Context::getContext()->smarty;
		
		// assign parameters
		$smarty->assign([
			'depo' => [
				'flashes' => $flashes,
				'order_depo' => $order_depo,
				'base_link' => Context::getContext()->link->getAdminLink('AdminDepoCarrier')
					. '&id_order=' . $order_depo->id,
				'history' => OrderDepoExtHistory::getByIdOrder($order_depo->id)
			]
		]);
		
		// render snippets
		return [
			'js-depo-shipping-content' => $smarty->fetch(_PS_MODULE_DIR_
				.'/depocarrier/views/templates/snippets/order_content_ship.tpl'),
			'js-depo-shipping-tab' => $smarty->fetch(_PS_MODULE_DIR_
				.'/depocarrier/views/templates/snippets/order_tab_ship.tpl')
		
		];
	}

	/**
	 * Outputs PDF
	 */
	public function processGeneratePDF() {
		$order_depo = new OrderDepoExt($this->id_order);
		
		$label = new \DEPOClient\Entities\Label();
		$label->number = $order_depo->number;
		
		$label->size = Tools::getValue('size', '2x2');
		$this->module->getClient()->generateLabel($label);

		header('Content-type: application/pdf');
		die(file_get_contents($label->pdf_filename));
	}

	/**
	 * Cancels a package
	 */
	public function ajaxProcessCancelPackage() {
		if ($this->checkToken()) {
			$order_depo = new OrderDepoExt($this->id_order);
			$result = $this->module->cancelPackage($order_depo);
			die(json_encode([
				'result' => $result ? 'success' : 'fail',
				'snippets' => self::renderSnippets($order_depo)
			]));
		}
	}
}